@@ -24,20 +24,21 @@
 guint mojito_debug_flags;
 
 void
-mojito_debug_init (void)
+mojito_debug_init (const char *string)
 {
   static gboolean setup_done = FALSE;
   static const GDebugKey keys[] = {
     { "main-loop", MOJITO_DEBUG_MAIN_LOOP },
     { "views", MOJITO_DEBUG_VIEWS },
-    { "online", MOJITO_DEBUG_ONLINE }
+    { "online", MOJITO_DEBUG_ONLINE },
+    { "item", MOJITO_DEBUG_ITEM },
+    { "twitter", MOJITO_DEBUG_TWITTER }
   };
 
   if (G_LIKELY (setup_done))
     return;
 
-  mojito_debug_flags = g_parse_debug_string (g_getenv ("MOJITO_DEBUG"),
-                                             keys, G_N_ELEMENTS (keys));
+  mojito_debug_flags = g_parse_debug_string (string, keys, G_N_ELEMENTS (keys));
 
   setup_done = TRUE;
 }
