@@ -21,7 +21,9 @@
 typedef enum {
   MOJITO_DEBUG_MAIN_LOOP = 1 << 0,
   MOJITO_DEBUG_VIEWS = 1 << 1,
-  MOJITO_DEBUG_ONLINE = 1 << 2
+  MOJITO_DEBUG_ONLINE = 1 << 2,
+  MOJITO_DEBUG_ITEM = 1 << 3,
+  MOJITO_DEBUG_TWITTER = 1 << 4
 } MojitoDebugFlags;
 
 extern guint mojito_debug_flags;
@@ -33,4 +35,4 @@ extern guint mojito_debug_flags;
       { g_message ("[" #category "] " G_STRLOC ": " x, ##a); }        \
   } G_STMT_END
 
-void mojito_debug_init (void);
+void mojito_debug_init (const char *string);
