@@ -10,6 +10,7 @@ struct _MojitoServiceIfaceClass {
     mojito_service_iface_get_static_capabilities_impl get_static_capabilities;
     mojito_service_iface_get_dynamic_capabilities_impl get_dynamic_capabilities;
     mojito_service_iface_request_avatar_impl request_avatar;
+    mojito_service_iface_credentials_updated_impl credentials_updated;
 };
 
 enum {
@@ -228,6 +229,50 @@ mojito_service_iface_implement_request_avatar (MojitoServiceIfaceClass *klass, m
   klass->request_avatar = impl;
 }
 
+/**
+ * mojito_service_iface_credentials_updated_impl:
+ * @self: The object implementing this interface
+ * @context: Used to return values or throw an error
+ *
+ * The signature of an implementation of the D-Bus method
+ * CredentialsUpdated on interface com.intel.Mojito.Service.
+ */
+static void
+mojito_service_iface_credentials_updated (MojitoServiceIface *self,
+    DBusGMethodInvocation *context)
+{
+  mojito_service_iface_credentials_updated_impl impl = (MOJITO_SERVICE_IFACE_GET_CLASS (self)->credentials_updated);
+
+  if (impl != NULL)
+    {
+      (impl) (self,
+        context);
+    }
+  else
+    {
+      GError e = { DBUS_GERROR, 
+           DBUS_GERROR_UNKNOWN_METHOD,
+           "Method not implemented" };
+
+      dbus_g_method_return_error (context, &e);
+    }
+}
+
+/**
+ * mojito_service_iface_implement_credentials_updated:
+ * @klass: A class whose instances implement this interface
+ * @impl: A callback used to implement the CredentialsUpdated D-Bus method
+ *
+ * Register an implementation for the CredentialsUpdated method in the vtable
+ * of an implementation of this interface. To be called from
+ * the interface init function.
+ */
+void
+mojito_service_iface_implement_credentials_updated (MojitoServiceIfaceClass *klass, mojito_service_iface_credentials_updated_impl impl)
+{
+  klass->credentials_updated = impl;
+}
+
 /**
  * mojito_service_iface_emit_status_updated:
  * @instance: The object implementing this interface
@@ -394,13 +439,14 @@ static const DBusGMethodInfo _mojito_service_iface_methods[] = {
   { (GCallback) mojito_service_iface_get_static_capabilities, g_cclosure_marshal_VOID__POINTER, 60 },
   { (GCallback) mojito_service_iface_get_dynamic_capabilities, g_cclosure_marshal_VOID__POINTER, 124 },
   { (GCallback) mojito_service_iface_request_avatar, g_cclosure_marshal_VOID__POINTER, 189 },
+  { (GCallback) mojito_service_iface_credentials_updated, g_cclosure_marshal_VOID__POINTER, 231 },
 };
 
 static const DBusGObjectInfo _mojito_service_iface_object_info = {
   0,
   _mojito_service_iface_methods,
-  4,
-"com.intel.Mojito.Service\0UpdateStatus\0A\0status_message\0I\0s\0\0com.intel.Mojito.Service\0GetStaticCapabilities\0A\0caps\0O\0F\0N\0as\0\0com.intel.Mojito.Service\0GetDynamicCapabilities\0A\0caps\0O\0F\0N\0as\0\0com.intel.Mojito.Service\0RequestAvatar\0A\0\0\0",
+  5,
+"com.intel.Mojito.Service\0UpdateStatus\0A\0status_message\0I\0s\0\0com.intel.Mojito.Service\0GetStaticCapabilities\0A\0caps\0O\0F\0N\0as\0\0com.intel.Mojito.Service\0GetDynamicCapabilities\0A\0caps\0O\0F\0N\0as\0\0com.intel.Mojito.Service\0RequestAvatar\0A\0\0com.intel.Mojito.Service\0CredentialsUpdated\0A\0\0\0",
 "com.intel.Mojito.Service\0StatusUpdated\0com.intel.Mojito.Service\0CapabilitiesChanged\0com.intel.Mojito.Service\0AvatarRetrieved\0com.intel.Mojito.Service\0UserChanged\0\0",
 "\0\0",
 };
