@@ -113,6 +113,25 @@ mojito_service_iface_return_from_request_avatar (DBusGMethodInvocation *context)
   dbus_g_method_return (context);
 }
 
+typedef void (*mojito_service_iface_credentials_updated_impl) (MojitoServiceIface *self,
+    DBusGMethodInvocation *context);
+void mojito_service_iface_implement_credentials_updated (MojitoServiceIfaceClass *klass, mojito_service_iface_credentials_updated_impl impl);
+/**
+ * mojito_service_iface_return_from_credentials_updated:
+ * @context: The D-Bus method invocation context
+ *
+ * Return successfully by calling dbus_g_method_return().
+ * This inline function exists only to provide type-safety.
+ */
+static inline
+/* this comment is to stop gtkdoc realising this is static */
+void mojito_service_iface_return_from_credentials_updated (DBusGMethodInvocation *context);
+static inline void
+mojito_service_iface_return_from_credentials_updated (DBusGMethodInvocation *context)
+{
+  dbus_g_method_return (context);
+}
+
 void mojito_service_iface_emit_status_updated (gpointer instance,
     gboolean arg_success);
 void mojito_service_iface_emit_capabilities_changed (gpointer instance,
