@@ -116,13 +116,23 @@ munge_items (MojitoView *view)
   int count, service_max;
   GHashTable *counts;
   MojitoSet *new;
+  MojitoItem *item;
 
   /* The magic */
   list = mojito_set_as_list (priv->all_items);
 
   while (list) {
-    if (!mojito_core_is_item_banned (priv->core, (MojitoItem *)list->data)) {
-      l = g_list_prepend (l, list->data);
+    item = (MojitoItem *)list->data;
+    if (!mojito_core_is_item_banned (priv->core, item)) {
+
+      if (mojito_item_get_ready (item)) {
+        l = g_list_prepend (l, item);
+        MOJITO_DEBUG (VIEWS, "Item ready: %s",
+                      mojito_item_get (item, "id"));
+      } else {
+        MOJITO_DEBUG (VIEWS, "Item not ready. Skipping: %s",
+                      mojito_item_get (item, "id"));
+      }
     }
     list = g_list_delete_link (list, list);
   }
@@ -229,6 +239,40 @@ mojito_view_recalculate (MojitoView *view)
   priv->current = new_items;
 }
 
+static void
+_item_ready_notify_cb (MojitoItem *item,
+                       GParamSpec *pspec,
+                       MojitoView *view)
+{
+  /* TODO: Use a timeout to rate limit this */
+  if (mojito_item_get_ready (item)) {
+    MOJITO_DEBUG (VIEWS, "Item became ready: %s.",
+                  mojito_item_get (item, "id"));
+    mojito_view_recalculate (view);
+    g_signal_handlers_disconnect_by_func (item,
+                                          _item_ready_notify_cb,
+                                          view);
+  }
+}
+
+static void
+setup_ready_handler (gpointer object, gpointer user_data)
+{
+  MojitoItem *item = MOJITO_ITEM (object);
+  MojitoView *view = MOJITO_VIEW (user_data);
+
+  if (mojito_item_get_ready (item))
+    return;
+
+  MOJITO_DEBUG (VIEWS, "Item not ready. Setting up signal handler: %s.",
+                mojito_item_get (item, "id"));
+
+  g_signal_connect (item,
+                    "notify::ready",
+                    (GCallback)_item_ready_notify_cb,
+                    view);
+}
+
 static void
 service_updated (MojitoService *service, MojitoSet *set, gpointer user_data)
 {
@@ -245,6 +289,7 @@ service_updated (MojitoService *service, MojitoSet *set, gpointer user_data)
   mojito_set_foreach_remove (priv->all_items, remove_service, service);
   if (set) {
     mojito_set_add_from (priv->all_items, set);
+    mojito_set_foreach (set, setup_ready_handler, view);
     mojito_set_unref (set);
   }
 
@@ -548,6 +593,7 @@ test_view_new (void)
   g_object_unref (view);
 }
 
+#if 0
 static MojitoItem *
 make_item_dated (MojitoService *service, time_t date)
 {
@@ -572,6 +618,7 @@ make_item (MojitoService *service)
 {
   return make_item_dated (service, time (NULL));
 }
+#endif
 
 /*
  * Create a view of 5 items on a single service, add 5 items to the pending
