@@ -108,9 +108,9 @@ get_utc_date (const char *s)
 
 static void
 got_status_cb (RestProxyCall *call,
-             GError        *error,
-             GObject       *weak_object,
-             gpointer       userdata)
+               const GError  *error,
+               GObject       *weak_object,
+               gpointer       userdata)
 {
   MojitoService *service = MOJITO_SERVICE (weak_object);
   MojitoServiceMySpacePrivate *priv = MOJITO_SERVICE_MYSPACE (service)->priv;
@@ -179,7 +179,7 @@ got_status_cb (RestProxyCall *call,
     /* TODO: async downloading */
     subnode = rest_xml_node_find (node, "imageurl");
     if (subnode && subnode->content)
-      mojito_item_take (item, "authoricon", mojito_web_download_image (subnode->content));
+      mojito_item_request_image_fetch (item, "authoricon", subnode->content);
 
     mojito_item_put (item, "content", rest_xml_node_find (node, "status")->content);
     /* TODO: if mood is not "(none)" then append that to the status message */
@@ -280,7 +280,7 @@ get_dynamic_caps (MojitoService *service)
   };
   static const char * no_caps[] = { NULL };
 
-  if (myspace->priv->proxy)
+  if (myspace->priv->user_id)
     return caps;
   else
     return no_caps;
@@ -288,7 +288,7 @@ get_dynamic_caps (MojitoService *service)
 
 static void
 got_user_cb (RestProxyCall *call,
-             GError        *error,
+             const GError  *error,
              GObject       *weak_object,
              gpointer       userdata)
 {
@@ -388,7 +388,7 @@ request_avatar (MojitoService *service)
 
 static void
 _status_updated_cb (RestProxyCall *call,
-                    GError        *error,
+                    const GError  *error,
                     GObject       *weak_object,
                     gpointer       userdata)
 {
@@ -446,6 +446,9 @@ online_notify (gboolean online, gpointer user_data)
       g_object_unref (priv->proxy);
       priv->proxy = NULL;
     }
+
+    g_free (priv->user_id);
+    priv->user_id = NULL;
   }
 }
 
