@@ -18,6 +18,7 @@
 
 #include <config.h>
 #include <time.h>
+#include <stdlib.h>
 #include <string.h>
 #include "twitter.h"
 #include <mojito/mojito-item.h>
@@ -25,8 +26,10 @@
 #include <mojito/mojito-online.h>
 #include <mojito/mojito-utils.h>
 #include <mojito/mojito-web.h>
+#include <mojito/mojito-debug.h>
 #include <mojito-keyfob/mojito-keyfob.h>
 #include <mojito-keystore/mojito-keystore.h>
+#include <gconf/gconf-client.h>
 #include <rest/oauth-proxy.h>
 #include <rest/rest-xml-parser.h>
 #include <libsoup/soup.h>
@@ -36,6 +39,8 @@ G_DEFINE_TYPE (MojitoServiceTwitter, mojito_service_twitter, MOJITO_TYPE_SERVICE
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MOJITO_TYPE_SERVICE_TWITTER, MojitoServiceTwitterPrivate))
 
+#define TWITTER_USE_OAUTH 0
+
 struct _MojitoServiceTwitterPrivate {
   enum {
     OWN,
@@ -46,8 +51,47 @@ struct _MojitoServiceTwitterPrivate {
   char *user_id;
   char *image_url;
   GRegex *twitpic_re;
+#if ! TWITTER_USE_OAUTH
+  GConfClient *gconf;
+  guint gconf_notify_id[2];
+  char *username, *password;
+#endif
 };
 
+#define KEY_BASE "/apps/mojito/services/twitter"
+#define KEY_USER KEY_BASE "/user"
+#define KEY_PASS KEY_BASE "/password"
+
+#if ! TWITTER_USE_OAUTH
+static void online_notify (gboolean online, gpointer user_data);
+static void credentials_updated (MojitoService *service);
+
+static void
+auth_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+  MojitoService *service = MOJITO_SERVICE (user_data);
+  MojitoServiceTwitter *twitter = MOJITO_SERVICE_TWITTER (service);
+  MojitoServiceTwitterPrivate *priv = twitter->priv;
+  const char *username = NULL, *password = NULL;
+
+  if (g_str_equal (entry->key, KEY_USER)) {
+    if (entry->value)
+      username = gconf_value_get_string (entry->value);
+    if (username && username[0] == '\0')
+      username = NULL;
+    priv->username = g_strdup (username);
+  } else if (g_str_equal (entry->key, KEY_PASS)) {
+    if (entry->value)
+      password = gconf_value_get_string (entry->value);
+    if (password && password[0] == '\0')
+      password = NULL;
+    priv->password = g_strdup (password);
+  }
+
+  credentials_updated (service);
+}
+#endif
+
 RestXmlNode *
 node_from_call (RestProxyCall *call)
 {
@@ -71,8 +115,6 @@ node_from_call (RestProxyCall *call)
                                           rest_proxy_call_get_payload (call),
                                           rest_proxy_call_get_payload_length (call));
 
-  g_object_unref (call);
-
   if (root == NULL) {
     g_message ("Error from Twitter: %s",
                rest_proxy_call_get_payload (call));
@@ -90,6 +132,33 @@ make_date (const char *s)
   return mojito_time_t_to_string (timegm (&tm));
 }
 
+/*
+ * Remove trailing and leading whitespace and hyphens in an attempt to clean up
+ * twitpic tweets.
+ */
+static void
+cleanup_twitpic (char *string)
+{
+  guchar *start;
+  size_t len;
+
+  g_return_if_fail (string != NULL);
+
+  for (start = (guchar*) string; *start && (g_ascii_isspace (*start) || *start == '-'); start++)
+    ;
+
+  len = strlen ((char*)start);
+
+  g_memmove (string, start, len + 1);
+
+  while (len--) {
+    if (g_ascii_isspace ((guchar) string[len]) || string[len] == '-')
+      string[len] = '\0';
+    else
+      break;
+  }
+}
+
 static MojitoItem *
 make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
 {
@@ -128,13 +197,16 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
     /* Construct the thumbnail URL and download the image */
     twitpic_id = g_match_info_fetch (match_info, 1);
     url = g_strconcat ("http://twitpic.com/show/thumb/", twitpic_id, NULL);
-    mojito_item_take (item, "thumbnail", mojito_web_download_image (url));
+    mojito_item_request_image_fetch (item, "thumbnail", url);
     g_free (url);
 
     /* Remove the URL from the tweet and use that as the title */
     new_content = g_regex_replace (priv->twitpic_re,
                                    content, -1,
                                    0, "", 0, NULL);
+
+    cleanup_twitpic (new_content);
+
     mojito_item_take (item, "title", new_content);
 
     /* Update the URL to point at twitpic */
@@ -156,14 +228,15 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
 
   n = rest_xml_node_find (u_node, "profile_image_url");
   if (n && n->content)
-    mojito_item_take (item, "authoricon", mojito_web_download_image (n->content));
+    mojito_item_request_image_fetch (item, "authoricon", n->content);
+
 
   return item;
 }
 
 static void
 tweets_cb (RestProxyCall *call,
-           GError        *error,
+           const GError  *error,
            GObject       *weak_object,
            gpointer       userdata)
 {
@@ -182,6 +255,8 @@ tweets_cb (RestProxyCall *call,
 
   set = mojito_item_set_new ();
 
+  MOJITO_DEBUG (TWITTER, "Got tweets!");
+
   for (node = rest_xml_node_find (root, "status"); node; node = node->next) {
     MojitoItem *item;
     /* TODO: skip the user's own tweets */
@@ -207,6 +282,8 @@ get_status_updates (MojitoServiceTwitter *twitter)
   if (!priv->user_id || !priv->running)
     return;
 
+  MOJITO_DEBUG (TWITTER, "Got status updates");
+
   call = rest_proxy_new_call (priv->proxy);
   switch (priv->type) {
   case OWN:
@@ -248,9 +325,35 @@ get_dynamic_caps (MojitoService *service)
     return no_caps;
 }
 
+static void
+sanity_check_date (RestProxyCall *call)
+{
+  GHashTable *headers;
+  SoupDate *call_date;
+  const char *s;
+  time_t call_time, diff;
+
+  headers = rest_proxy_call_get_response_headers (call);
+  s = g_hash_table_lookup (headers, "Date");
+  if (s) {
+    call_date = soup_date_new_from_string (s);
+    if (call_date) {
+      call_time = soup_date_to_time_t (call_date);
+      diff = labs (time (NULL) - call_time);
+      /* More than five minutes difference between local time and the response
+         time? */
+      if (diff > (60 * 5)) {
+        g_warning ("%ld seconds difference between HTTP time and system time!", diff);
+      }
+    }
+    soup_date_free (call_date);
+  }
+  g_hash_table_unref (headers);
+}
+
 static void
 verify_cb (RestProxyCall *call,
-           GError        *error,
+           const GError  *error,
            GObject       *weak_object,
            gpointer       userdata)
 {
@@ -259,10 +362,13 @@ verify_cb (RestProxyCall *call,
   RestXmlNode *node;
 
   if (error) {
+    sanity_check_date (call);
     g_message ("Error: %s", error->message);
     return;
   }
 
+  MOJITO_DEBUG (TWITTER, "Authentication verified");
+
   node = node_from_call (call);
   if (!node)
     return;
@@ -286,6 +392,7 @@ got_tokens_cb (RestProxy *proxy, gboolean authorised, gpointer user_data)
   RestProxyCall *call;
 
   if (authorised) {
+    MOJITO_DEBUG (TWITTER, "Authorised");
     call = rest_proxy_new_call (priv->proxy);
     rest_proxy_call_set_function (call, "account/verify_credentials.xml");
     rest_proxy_call_async (call, verify_cb, (GObject*)twitter, NULL, NULL);
@@ -311,16 +418,23 @@ refresh (MojitoService *service)
   if (!priv->running)
     return;
 
+#if TWITTER_USE_OAUTH
   if (priv->user_id) {
     get_status_updates (twitter);
   } else {
     mojito_keyfob_oauth ((OAuthProxy*)priv->proxy, got_tokens_cb, service);
   }
+#else
+  if (priv->username && priv->password && priv->proxy)
+  {
+    got_tokens_cb (priv->proxy, TRUE, twitter);
+  }
+#endif
 }
 
 static void
 _status_updated_cb (RestProxyCall *call,
-                    GError        *error,
+                    const GError  *error,
                     GObject       *weak_object,
                     gpointer       userdata)
 {
@@ -378,14 +492,43 @@ online_notify (gboolean online, gpointer user_data)
 {
   MojitoServiceTwitter *twitter = (MojitoServiceTwitter *)user_data;
   MojitoServiceTwitterPrivate *priv = twitter->priv;
-  const char *key = NULL, *secret = NULL;
+
+  MOJITO_DEBUG (TWITTER, "Online: %s", online ? "yes" : "no");
 
   if (online) {
+#if TWITTER_USE_OAUTH
+    const char *key = NULL, *secret = NULL;
+
     mojito_keystore_get_key_secret ("twitter", &key, &secret);
-
     priv->proxy = oauth_proxy_new (key, secret, "http://twitter.com/", FALSE);
-
     mojito_keyfob_oauth ((OAuthProxy *)priv->proxy, got_tokens_cb, twitter);
+#else
+    if (priv->username && priv->password) {
+      char *url;
+      char *escaped_user;
+      char *escaped_password;
+
+      escaped_user = g_uri_escape_string (priv->username,
+                                          NULL,
+                                          FALSE);
+      escaped_password = g_uri_escape_string (priv->password,
+                                          NULL,
+                                          FALSE);
+
+      url = g_strdup_printf ("https://%s:%s@twitter.com/",
+                             escaped_user, escaped_password);
+
+      g_free (escaped_user);
+      g_free (escaped_password);
+
+      priv->proxy = rest_proxy_new (url, FALSE);
+      g_free (url);
+
+      got_tokens_cb (priv->proxy, TRUE, twitter);
+    } else {
+      mojito_service_emit_refreshed ((MojitoService *)twitter, NULL);
+    }
+#endif
   } else {
     if (priv->proxy) {
       g_object_unref (priv->proxy);
@@ -398,6 +541,19 @@ online_notify (gboolean online, gpointer user_data)
   }
 }
 
+static void
+credentials_updated (MojitoService *service)
+{
+  MOJITO_DEBUG (TWITTER, "Credentials updated");
+
+  /* If we're online, force a reconnect to fetch new credentials */
+  if (mojito_is_online ()) {
+    online_notify (FALSE, service);
+    online_notify (TRUE, service);
+  }
+
+  mojito_service_emit_user_changed (service);
+}
 
 static const char *
 mojito_service_twitter_get_name (MojitoService *service)
@@ -422,6 +578,20 @@ mojito_service_twitter_constructed (GObject *object)
   priv->twitpic_re = g_regex_new ("http://twitpic.com/([A-Za-z0-9]+)", 0, 0, NULL);
   g_assert (priv->twitpic_re);
 
+#if ! TWITTER_USE_OAUTH
+  priv->gconf = gconf_client_get_default ();
+  gconf_client_add_dir (priv->gconf, KEY_BASE,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  priv->gconf_notify_id[0] = gconf_client_notify_add (priv->gconf, KEY_USER,
+                                                      auth_changed_cb, twitter,
+                                                      NULL, NULL);
+  priv->gconf_notify_id[1] = gconf_client_notify_add (priv->gconf, KEY_PASS,
+                                                      auth_changed_cb, twitter,
+                                                      NULL, NULL);
+  gconf_client_notify (priv->gconf, KEY_USER);
+  gconf_client_notify (priv->gconf, KEY_PASS);
+#endif
+
   mojito_online_add_notify (online_notify, twitter);
   if (mojito_is_online ()) {
     online_notify (TRUE, twitter);
@@ -445,6 +615,15 @@ mojito_service_twitter_dispose (GObject *object)
     priv->twitpic_re = NULL;
   }
 
+#if ! TWITTER_USE_OAUTH
+  if (priv->gconf) {
+    gconf_client_notify_remove (priv->gconf, priv->gconf_notify_id[0]);
+    gconf_client_notify_remove (priv->gconf, priv->gconf_notify_id[1]);
+    g_object_unref (priv->gconf);
+    priv->gconf = NULL;
+  }
+#endif
+
   G_OBJECT_CLASS (mojito_service_twitter_parent_class)->dispose (object);
 }
 
@@ -456,6 +635,11 @@ mojito_service_twitter_finalize (GObject *object)
   g_free (priv->user_id);
   g_free (priv->image_url);
 
+#if ! TWITTER_USE_OAUTH
+  g_free (priv->username);
+  g_free (priv->password);
+#endif
+
   G_OBJECT_CLASS (mojito_service_twitter_parent_class)->finalize (object);
 }
 
@@ -478,9 +662,11 @@ mojito_service_twitter_class_init (MojitoServiceTwitterClass *klass)
   service_class->get_dynamic_caps = get_dynamic_caps;
   service_class->update_status = update_status;
   service_class->request_avatar = request_avatar;
+  service_class->credentials_updated = credentials_updated;
 }
 
 static void
 mojito_service_twitter_init (MojitoServiceTwitter *self)
 {
+  self->priv = GET_PRIVATE (self);
 }
